summaryrefslogtreecommitdiff
path: root/tools/testing
diff options
context:
space:
mode:
authorBen Gardon <bgardon@google.com>2020-01-23 10:04:30 -0800
committerPaolo Bonzini <pbonzini@redhat.com>2020-03-16 17:57:02 +0100
commitaf99e1ad7e708d1a1a4e4c1bb10a2b851974fc04 (patch)
tree96d00db6c5cd5feaa522878339c8bd4085e31bb7 /tools/testing
parent0119cb365c93621535187c7527486c3b378a622d (diff)
KVM: selftests: Add memory size parameter to the demand paging test
Add an argument to allow the demand paging test to work on larger and smaller guest sizes. Signed-off-by: Ben Gardon <bgardon@google.com> [Rewrote parse_size() to simplify and provide user more flexibility as to how sizes are input. Also fixed size overflow assert.] Signed-off-by: Andrew Jones <drjones@redhat.com> Signed-off-by: Paolo Bonzini <pbonzini@redhat.com>
Diffstat (limited to 'tools/testing')
-rw-r--r--tools/testing/selftests/kvm/Makefile2
-rw-r--r--tools/testing/selftests/kvm/demand_paging_test.c57
-rw-r--r--tools/testing/selftests/kvm/include/test_util.h2
-rw-r--r--tools/testing/selftests/kvm/lib/test_util.c51
4 files changed, 90 insertions, 22 deletions
diff --git a/tools/testing/selftests/kvm/Makefile b/tools/testing/selftests/kvm/Makefile
index 1bc9f41d3fcd..1bda24d30b3a 100644
--- a/tools/testing/selftests/kvm/Makefile
+++ b/tools/testing/selftests/kvm/Makefile
@@ -7,7 +7,7 @@ top_srcdir = ../../../..
KSFT_KHDR_INSTALL := 1
UNAME_M := $(shell uname -m)
-LIBKVM = lib/assert.c lib/elf.c lib/io.c lib/kvm_util.c lib/sparsebit.c
+LIBKVM = lib/assert.c lib/elf.c lib/io.c lib/kvm_util.c lib/sparsebit.c lib/test_util.c
LIBKVM_x86_64 = lib/x86_64/processor.c lib/x86_64/vmx.c lib/x86_64/svm.c lib/x86_64/ucall.c
LIBKVM_aarch64 = lib/aarch64/processor.c lib/aarch64/ucall.c
LIBKVM_s390x = lib/s390x/processor.c lib/s390x/ucall.c
diff --git a/tools/testing/selftests/kvm/include/test_util.h b/tools/testing/selftests/kvm/include/test_util.h
index a41db6fb7e24..e696c8219d69 100644
--- a/tools/testing/selftests/kvm/include/test_util.h
+++ b/tools/testing/selftests/kvm/include/test_util.h
@@ -39,4 +39,6 @@ void test_assert(bool exp, const char *exp_str,
#a, #b, #a, (unsigned long) __a, #b, (unsigned long) __b); \
} while (0)
+size_t parse_size(const char *size);
+
#endif /* SELFTEST_KVM_TEST_UTIL_H */
diff --git a/tools/testing/selftests/kvm/lib/test_util.c b/tools/testing/selftests/kvm/lib/test_util.c
new file mode 100644
index 000000000000..cbd7f51b07a1
--- /dev/null
+++ b/tools/testing/selftests/kvm/lib/test_util.c
@@ -0,0 +1,51 @@
+// SPDX-License-Identifier: GPL-2.0-only
+/*
+ * tools/testing/selftests/kvm/lib/test_util.c
+ *
+ * Copyright (C) 2020, Google LLC.
+ */
+#include <stdlib.h>
+#include <ctype.h>
+#include <limits.h>
+#include "test_util.h"
+
+/*
+ * Parses "[0-9]+[kmgt]?".
+ */
+size_t parse_size(const char *size)
+{
+ size_t base;
+ char *scale;
+ int shift = 0;
+
+ TEST_ASSERT(size && isdigit(size[0]), "Need at least one digit in '%s'", size);
+
+ base = strtoull(size, &scale, 0);
+
+ TEST_ASSERT(base != ULLONG_MAX, "Overflow parsing size!");
+
+ switch (tolower(*scale)) {
+ case 't':
+ shift = 40;
+ break;
+ case 'g':
+ shift = 30;
+ break;
+ case 'm':
+ shift = 20;
+ break;
+ case 'k':
+ shift = 10;
+ break;
+ case 'b':
+ case '\0':
+ shift = 0;
+ break;
+ default:
+ TEST_ASSERT(false, "Unknown size letter %c", *scale);
+ }
+
+ TEST_ASSERT((base << shift) >> shift == base, "Overflow scaling size!");
+
+ return base << shift;
+}
diff --git a/tools/testing/selftests/kvm/demand_paging_test.c b/tools/testing/selftests/kvm/demand_paging_test.c
index ab302e1f5230..c1880b3e3041 100644
--- a/tools/testing/selftests/kvm/demand_paging_test.c
+++ b/tools/testing/selftests/kvm/demand_paging_test.c
@@ -34,6 +34,8 @@
/* Default guest test virtual memory offset */
#define DEFAULT_GUEST_TEST_MEM 0xc0000000
+#define DEFAULT_GUEST_TEST_MEM_SIZE (1 << 30) /* 1G */
+
/*
* Guest/Host shared variables. Ensure addr_gva2hva() and/or
* sync_global_to/from_guest() are used when accessing from
@@ -276,11 +278,10 @@ static int setup_demand_paging(struct kvm_vm *vm,
return 0;
}
-#define GUEST_MEM_SHIFT 30 /* 1G */
#define PAGE_SHIFT_4K 12
static void run_test(enum vm_guest_mode mode, bool use_uffd,
- useconds_t uffd_delay)
+ useconds_t uffd_delay, uint64_t guest_memory_bytes)
{
pthread_t vcpu_thread;
pthread_t uffd_handler_thread;
@@ -289,33 +290,40 @@ static void run_test(enum vm_guest_mode mode, bool use_uffd,
int r;
/*
* We reserve page table for 2 times of extra dirty mem which
* will definitely cover the original (1G+) test range. Here
* we do the calculation with 4K page size which is the
* smallest so the page number will be enough for all archs
* (e.g., 64K page size guest will need even less memory for
* page tables).
*/
vm = create_vm(mode, VCPU_ID,
- 2ul << (GUEST_MEM_SHIFT - PAGE_SHIFT_4K),
+ (2 * guest_memory_bytes) >> PAGE_SHIFT_4K,
guest_code);
guest_page_size = vm_get_page_size(vm);
- /*
- * A little more than 1G of guest page sized pages. Cover the
- * case where the size is not aligned to 64 pages.
- */
- guest_num_pages = (1ul << (GUEST_MEM_SHIFT -
- vm_get_page_shift(vm))) + 16;
+
+ TEST_ASSERT(guest_memory_bytes % guest_page_size == 0,
+ "Guest memory size is not guest page size aligned.");
+
+ guest_num_pages = guest_memory_bytes / guest_page_size;
+
#ifdef __s390x__
/* Round up to multiple of 1M (segment size) */
guest_num_pages = (guest_num_pages + 0xff) & ~0xffUL;
#endif
+ /*
+ * If there should be more memory in the guest test region than there
+ * can be pages in the guest, it will definitely cause problems.
+ */
+ TEST_ASSERT(guest_num_pages < vm_get_max_gfn(vm),
+ "Requested more guest memory than address space allows.\n"
+ " guest pages: %lx max gfn: %lx\n",
+ guest_num_pages, vm_get_max_gfn(vm));
host_page_size = getpagesize();
- host_num_pages = (guest_num_pages * guest_page_size) / host_page_size +
- !!((guest_num_pages * guest_page_size) %
- host_page_size);
+ TEST_ASSERT(guest_memory_bytes % host_page_size == 0,
+ "Guest memory size is not host page size aligned.");
+ host_num_pages = guest_memory_bytes / host_page_size;
guest_test_phys_mem = (vm_get_max_gfn(vm) - guest_num_pages) *
guest_page_size;
@@ -403,7 +411,8 @@ static void help(char *name)
int i;
puts("");
- printf("usage: %s [-h] [-m mode] [-u] [-d uffd_delay_usec]\n", name);
+ printf("usage: %s [-h] [-m mode] [-u] [-d uffd_delay_usec]\n"
+ " [-b memory]\n", name);
printf(" -m: specify the guest mode ID to test\n"
" (default: test all supported modes)\n"
" This option may be used multiple times.\n"
@@ -417,6 +426,8 @@ static void help(char *name)
printf(" -d: add a delay in usec to the User Fault\n"
" FD handler to simulate demand paging\n"
" overheads. Ignored without -u.\n");
+ printf(" -b: specify the size of the memory region which should be\n"
+ " demand paged. e.g. 10M or 3G. Default: 1G\n");
puts("");
exit(0);
}
@@ -424,6 +435,7 @@ static void help(char *name)
int main(int argc, char *argv[])
{
bool mode_selected = false;
+ uint64_t guest_memory_bytes = DEFAULT_GUEST_TEST_MEM_SIZE;
unsigned int mode;
int opt, i;
bool use_uffd = false;
@@ -450,7 +462,7 @@ int main(int argc, char *argv[])
guest_mode_init(VM_MODE_P40V48_4K, true, true);
#endif
- while ((opt = getopt(argc, argv, "hm:ud:")) != -1) {
+ while ((opt = getopt(argc, argv, "hm:ud:b:")) != -1) {
switch (opt) {
case 'm':
if (!mode_selected) {
@@ -471,6 +483,9 @@ int main(int argc, char *argv[])
TEST_ASSERT(uffd_delay >= 0,
"A negative UFFD delay is not supported.");
break;
+ case 'b':
+ guest_memory_bytes = parse_size(optarg);
+ break;
case 'h':
default:
help(argv[0]);
@@ -484,7 +499,7 @@ int main(int argc, char *argv[])
TEST_ASSERT(guest_modes[i].supported,
"Guest mode ID %d (%s) not supported.",
i, vm_guest_mode_string(i));
- run_test(i, use_uffd, uffd_delay);
+ run_test(i, use_uffd, uffd_delay, guest_memory_bytes);
}
return 0;
@@ -289,33 +290,40 @@ static void run_test(enum vm_guest_mode mode, bool use_uffd,
int r;
/*
- * We reserve page table for 2 times of extra dirty mem which
- * will definitely cover the original (1G+) test range. Here
- * we do the calculation with 4K page size which is the
- * smallest so the page number will be enough for all archs
- * (e.g., 64K page size guest will need even less memory for
- * page tables).
+ * We reserve page table for twice the ammount of memory we intend
+ * to use in the test region for demand paging. Here we do the
+ * calculation with 4K page size which is the smallest so the page
+ * number will be enough for all archs. (e.g., 64K page size guest
+ * will need even less memory for page tables).
*/
vm = create_vm(mode, VCPU_ID,